#include <netconfig.h>
#include <netdir.h>
#include <tiuser.h>
#include <stdio.h>
#include "tpinfo.h"

struct tpinfo *
get_addrs(host, service)
char *host;
char *service;
{
   void *handlep;                    /* handle to routines*/
   struct netconfig *netconfigp;     /* provider info     */
   struct nd_hostserv nd_hostserv;   /* host/service name */
   struct nd_addrlist *nd_addrlistp; /* list of addrs     */
   struct tpinfo *headp;         /* start of tpinfo list  */
   struct tpinfo *currp;         /* traverses tpinfo list */
   struct tpinfo **currpp;       /* traverses tpinfo list */
   int got_it;                   /* indicates we found a  */
                                 /* transport provider    */

   if ((handlep = setnetpath()) == NULL) {
      nc_perror("get_addr");
      return(NULL);
   }

   /*
    *  Use the host name and service name given
    *  as an argument.
    */

   if (host == NULL) {
      nd_hostserv.h_host = HOST_SELF;
   } else {
      nd_hostserv.h_host = host;
   }
   nd_hostserv.h_serv = service;

   headp = NULL;
   currpp = &headp;

   while ((netconfigp = getnetpath(handlep)) != NULL) {
      /*
       *  We only want a connection-oriented transport
       *  provider here.
       */
      got_it = 0;
      switch (netconfigp->nc_semantics) {
         case NC_TPI_COTS_ORD:
         case NC_TPI_COTS:
            if (netdir_getbyname(netconfigp, &nd_hostserv,
                                    &nd_addrlistp) == 0) {
               got_it = 1;
               break;
            }
         default:
            break;
      }

      if (got_it) {
         if ((currp = *currpp = (struct tpinfo *)
                   malloc(sizeof(struct tpinfo))) == NULL) {
            fprintf(stderr, "Cannot allocate space\n");
            return(NULL);
         }

         /*
          *  Allocate space to hold the device name because
          *  endnetconfig() deallocates memory of netconfigp.
          */

         if ((currp->device = (char *)malloc(
              strlen(netconfigp->nc_device) + 1)) == NULL) {
            fprintf(stderr, "Cannot allocate space\n");
            return(NULL);
         }
         strcpy(currp->device, netconfigp->nc_device);

         currp->nd_addrlistp = nd_addrlistp;
         currpp = &(currp->nextp);
      }
   }
   *currpp = NULL;
   endnetconfig(handlep);
   return(headp);
}

free_addrs(tpinfop)
struct tpinfo *tpinfop;
{
   struct tpinfo *savep;

   while (tpinfop) {
      free(tpinfop->device);
      netdir_free((char *)tpinfop->nd_addrlistp,
                          ND_ADDRLIST);
      savep = tpinfop;
      tpinfop = tpinfop->nextp;
      free(savep);
   }
}
