#include <stdio.h>
#include <tiuser.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>

#define NUM_TRIES   20
int timed_out;

#define SERV_PORT 5134
struct hostent *gethostbyname();

extern int t_errno;

main(argc, argv)
int argc;
char **argv;
{
   int fd;                  /* fd into transport provider */
   int flags = 0;           /* used to receive info       */
   char buf[BUFSIZ];        /* holds message from server  */
   struct t_unitdata *datap;/* used to call server        */
   int tries = NUM_TRIES;   /* number of tries to send    */
   int got_it = 0;          /* determines datagram receipt*/
   struct hostent *hp;      /* holds IP address of server */
   struct sockaddr_in myaddr;/* address that client uses  */
   struct sockaddr_in servaddr; /* the server's full addr */
   void handler();          /* handles time-out signals   */
   

   /*
    *  Check for proper usage
    */

   if (argc != 3) {
      fprintf(stderr,"Usage: %s host username\n", argv[0]);
      exit(2);
   }
   /*
    *  Open the transport provider.
    */

   if ((fd = t_open("/dev/udp", O_RDWR, NULL)) < 0) {
      t_error("t_open failed!");
      exit(1);
   }

   /*
    *  Bind to an arbitrary return address.
    */

   if (t_bind(fd, NULL, NULL) < 0) {
      t_error("t_bind failed!");
      exit(1);
   }
   /*
    *  Get memory to hold the datagram.
    */

   if ((datap = (struct t_unitdata *)t_alloc(fd, T_UNITDATA,
                                      T_ALL)) == NULL) {
      t_error("t_alloc failed!");
      exit(1);
   }

   /*
    *  Fill in the server's address and the data.
    */
   if (datap->udata.maxlen < (strlen(argv[2]) + 1)) {
      fprintf(stderr, "The user name %s is too large\n",
                       argv[2]);
      exit(1);
   }

   datap->udata.len = strlen(argv[2]) + 1;
   strcpy(datap->udata.buf, argv[2]);

   datap->addr.len = sizeof(struct sockaddr_in);
   memset((void *)&servaddr, 0, sizeof(struct sockaddr_in));
   servaddr.sin_family = AF_INET;
   servaddr.sin_port = htons(SERV_PORT);

   hp = gethostbyname(argv[1]);
   if (hp == 0) {
      fprintf(stderr, "could not obtain address of %s\n",
         argv[1]);
      return (-1);
   }

   (void)memcpy((caddr_t)&servaddr.sin_addr,
               hp->h_addr_list[0], hp->h_length);

   (void)memcpy(datap->addr.buf, &servaddr,
                                          sizeof(servaddr));
   while (tries --) {
      if (t_sndudata(fd, datap) == -1) {
         t_error("t_sndudata failed!");
         exit(1);
      }

      /*
       *   Allow 15 seconds for the response to arrive.
       */
      timed_out = 0;
      sigset(SIGALRM, handler);
      alarm(15);

      if (t_rcvudata(fd, datap, &flags) == 0) {
         alarm(0);
         got_it = 1;
         break;
      }
      /*
       *  We now know the t_rcvudata() routine failed.
       *  Check if we timed out.  If we did, we continue
       *  the loop and resend the request.
       */

      if (timed_out) {
         printf("Server not responding, retrying...\n");
         continue;
      }

      /*
       *  If the t_rcvudata() routine fails, check if the
       *  reason was because an error occurred on a previous
       *  datagram we sent (t_errno is set to TLOOK if that is
       *  the case).  If that's so, clear the error event
       *  with the t_rcvuderr() routine.
       */

      if (t_errno == TLOOK) {
         if (t_rcvuderr(fd, NULL) < 0) {
            t_error("could not clear error event!");
            exit(1);
         }
         continue;
      }

      t_error("t_rcvudata failed!");
      exit(1);
   }

   if (!got_it) {
      printf("failed %d times, exiting\n", NUM_TRIES);
      exit(1);
   }

   if (datap->udata.buf[0] == 'y')
      printf("%s is logged on server\n", argv[2]);
   else
      printf("%s is not logged on server\n", argv[2]);

   exit(0);
}

void
handler()
{
   timed_out = 1;
}
